unit DeskWinApi;

interface

uses Windows;

Type
  TDeskWinAPICommand = record
    CommandType: integer;
    X: integer;
    Y: integer;
    Z: integer;
    A: integer;
    Ex: integer;
    Ey: integer;
    rad: integer;
    SlewSPS: integer;
    EndSPS: integer;
    Secs: Double;
    Axis: Byte;
    Quad: Byte;
    Dir: Byte;
    Output: Byte;
    State: Byte;
    NumSteps: integer;
    bXSps: integer;
    bYSps: integer;
    eXSps: integer;
    eYSps: integer;
    bZSps: integer;
    eZSps: integer;
  end;

Const
  // Command types
  ARCDIRECTION = 1;
  ACCELCOM = 2;
  GETVERSION = 3;
  RESETCONTROL = 4;
  GETPOS = 5;
  PAUSECONTROL = 6;
  CONTINUECONTROL = 7;
  ENDMOVE = 8;
  EMERGENCYSTOP = 9;
  SETPOS = 10;
  Output = 11;
  CONTROLDELAY = 12;
  LINEARFEED = 13;
  RAPIDFEED = 14;
  CW = 15;
  CCW = 16;
  HOMECONTROL = 17;
  SETOVERRIDE = 18;
  SETOVERRIDEMAX = 19;
  INDEXFEED = 20;
  READINPUTS = 21;
  CHASEENCODER = 22;
  OUTPUTIMEDIATE = 24;

  // status
  DESKWIN_OK = 90;
  DESKWIN_IN_POSITION = 91;
  DESKWIN_BUFFERFULL = 92;
  DESKWIN_BEGIN_STATUS_BUFFERFULL = 202;
  DESKWIN_ENDSTRING = 204;
  DESKWIN_SYNTAXERROR = 205;
  DESKWIN_LIMITHIT = 207;
  DESKWIN_INVALID_CHECKSUM = 208;
  DESKWIN_UNKNOWN_COMMAND = 209;
  DESKWIN_BEGINSTATUS = 210;
  DESKWIN_BEGINSTRING = 211;
  DESKWIN_INVALIDSTART = 212;
  DESKWIN_PROBEHIT = 213;
  DESKWIN_IN_EMSTOP = 254;

  DESKWIN_COMMAND_IS_IN_PROGRESS = 214;
  DESKWIN_PORT_DOES_NOT_EXIST = 215;
  DESKWIN_PORT_IS_ALREADY_OPEN = 216;
  DESKWIN_TIMEDOUT = 217;
  DESKWIN_FAILED_TO_RESET = 218;
  DESKWIN_NOTINITED = 255;

  DESKWIN_2nd_OK = 235;
  DESKWIN_2nd_IN_POSITION = 236;
  DESKWIN_2nd_BUFFERFULL = 237;
  DESKWIN_2nd_BEGIN_STATUS_BUFFERFULL = 244;
  DESKWIN_2nd_ENDSTRING = 245;
  DESKWIN_2nd_SYNTAXERROR = 246;
  DESKWIN_2nd_LIMITHIT = 247;
  DESKWIN_2nd_INVALID_CHECKSUM = 248;
  DESKWIN_2nd_UNKNOWN_COMMAND = 249;
  DESKWIN_2nd_BEGINSTATUS = 250;
  DESKWIN_2nd_BEGINSTRING = 251;
  DESKWIN_2nd_INVALIDSTART = 212;
  DESKWIN_2nd_PROBEHIT = 213;
  DESKWIN_2nd_IN_EMSTOP = 255;

  DESKWIN_2nd_COMMAND_IS_IN_PROGRESS = 241;
  DESKWIN_2nd_PORT_DOES_NOT_EXIST = 242;
  DESKWIN_2nd_PORT_IS_ALREADY_OPEN = 243;
  DESKWIN_2nd_TIMEDOUT = 238;
  DESKWIN_2nd_FAILED_TO_RESET = 239;
  DESKWIN_2nd_NOTINITED = 240;

var
  rs274ngc_errors: array [0 .. 198] of String;

Function DeskWinSetArcDirection(var Direction: Byte): Byte; Stdcall; external 'DeskWinAPI.DLL' name 'DeskWinSetArcDirection';

Function DeskWinSetAccelleration(const ACCELL, MaxSPS, StartVel: integer; const AccellScale: Byte): Byte; StdCall;
  external 'DeskWinAPI.DLL' name 'DeskWinSetAccelleration';

Function DeskWinGetFirmwareVersion: Double; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinGetFirmwareVersion';

Function DeskWinReset(var ComPort: integer; const imeConstant, Mode: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinReset';

Function DeskWinGetPosition(var X, Y, Z, A: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinGetPosition';

Function DeskWinPause: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinPause';

Function DeskWinContinue: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinContinue';

Function DeskWinEndCurrentMove: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinEndCurrentMove';

Function DeskWinEmStop: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinEmStop';

Function DeskWinSetPosition(const X, Y, Z, A: integer; const Axis: Byte): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetPosition';

Function DeskWinSetOutput(const Output, State: Byte; const X, Y, Z, A: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetOutput';

Function DeskWinSetOutputImediate(const OutputState: Byte): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetOutputImediate';

Function DeskWinDelay(const seconds: Double; var X, Y, Z, A: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinDelay';

Function DeskWinIndexMove(const X, Y, Z, A, SlewSPS: integer; const Axis: Byte; var PosX, PosY, PosZ, PosA: integer): Byte; StdCall;
  external 'DeskWinAPI.DLL' name 'DeskWinIndexMove';

Function DeskWinLinearMove(const X, Y, Z, A, SlewSPS, EndSPS: integer; var PosX, PosY, PosZ, PosA: integer): Byte; StdCall;
  external 'DeskWinAPI.DLL' name 'DeskWinLinearMove';

Function DeskWinRapidMove(const X, Y, Z, A, SlewSPS, EndSPS: integer; var PosX, PosY, PosZ, PosA: integer): Byte; StdCall;
  external 'DeskWinAPI.DLL' name 'DeskWinRapidMove';

Function DeskWinQuadrantArcMoveCW(const rad, StartX, StartY, EndX, EndY, SlewSPS, EndSPS: integer; const Quad: Byte; var PosX, PosY, PosZ, PosA: integer): Byte;
  StdCall; external 'DeskWinAPI.DLL' name 'DeskWinQuadrantArcMoveCW';

Function DeskWinQuadrantArcMoveCCW(const rad, StartX, StartY, EndX, EndY, SlewSPS, EndSPS: integer; const Quad: Byte; var PosX, PosY, PosZ, PosA: integer)
  : Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinQuadrantArcMoveCCW';

Function DeskWinHome(const Axis, HomeDir: Byte; const HomeVel: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinHome';

Function DeskWinSetMaxSPS(const MaxSPS, NumAxis, Inverted, TagA, MaxEncCount: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetMaxSPS';

Function DeskWinSetFeedOverride(const Value: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetFeedOverride';

Function DeskWinSetPWMOutput(Const Value: Byte): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetPWMOutput';

Function DeskWinReadInputs: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinReadInputs';

Function DeskWinChaseEncoder(const EncDir, Dir, X, Y, Z, Enc: integer): Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinChaseEncoder';

Function DeskWinEndChaseEncoder: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinEndChaseEncoder';

Function DeskWinSetProbePolarity: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinSetProbePolarity';

Function DeskWinClose: Byte; StdCall; external 'DeskWinAPI.DLL' name 'DeskWinClose';

Function rs274ngc_ext_init(const IniFile: pAnsiChar): shortint; StdCall; external 'DeskWinAPI.DLL' name 'rs274ngc_ext_init';

Function rs274ngc_read_block(const block: pAnsiChar): shortint; StdCall; external 'DeskWinAPI.DLL' name 'rs274ngc_read_block';

Function rs274ngc_execute_block: integer; StdCall; external 'DeskWinAPI.DLL' name 'rs274ngc_execute_block';

procedure FlushBuffer; StdCall; external 'DeskWinAPI.DLL' name 'FlushBuffer';

implementation

begin
  rs274ngc_errors[0] := 'No error';
  rs274ngc_errors[1] := 'No error';
  rs274ngc_errors[2] := 'No error';
  rs274ngc_errors[3] := 'No error';
  rs274ngc_errors[4] := 'A file is already open';
  rs274ngc_errors[5] := 'All axes missing with g92';
  rs274ngc_errors[6] := 'All axes missing with motion code';
  rs274ngc_errors[7] := 'Arc radius too small to reach end point';
  rs274ngc_errors[8] := 'Argument to acos out of range';
  rs274ngc_errors[9] := 'Argument to asin out of range';
  rs274ngc_errors[10] := 'Attempt to divide by zero';
  rs274ngc_errors[11] := 'Attempt to raise negative to non integer power';
  rs274ngc_errors[12] := 'Bad character used';
  rs274ngc_errors[13] := 'Bad format unsigned integer';
  rs274ngc_errors[14] := 'Bad number format';
  rs274ngc_errors[15] := 'Bug Bad g code modal group 100';
  rs274ngc_errors[16] := 'Bug code not g0 or g1';
  rs274ngc_errors[17] := 'Bug code not g17 g18 or g19';
  rs274ngc_errors[18] := 'Bug code not g20 or g21';
  rs274ngc_errors[19] := 'Bug code not g28 or g30';
  rs274ngc_errors[20] := 'Bug code not g2 or g3';
  rs274ngc_errors[21] := 'Bug code not g40 g41 or g42';
  rs274ngc_errors[22] := 'Bug code not g43 or g49';
  rs274ngc_errors[23] := 'Bug code not g4 g10 g28 g30 g53 or g92 series';
  rs274ngc_errors[24] := 'Bug code not g61 g61 1 or g64';
  rs274ngc_errors[25] := 'Bug code not g90 or g91';
  rs274ngc_errors[26] := 'Bug code not g93 or g94';
  rs274ngc_errors[27] := 'Bug code not g98 or g99';
  rs274ngc_errors[28] := 'Bug code not in g92 series';
  rs274ngc_errors[29] := 'Bug code not in range g54 to g593';
  rs274ngc_errors[30] := 'Bug code not m0 m1 m2 m30 m60';
  rs274ngc_errors[31] := 'Bug distance Mode not g90 or g91';
  rs274ngc_errors[32] := 'Bug function should not have been called';
  rs274ngc_errors[33] := 'Bug in tool radius comp';
  rs274ngc_errors[34] := 'Bug plane not xy yz or xz';
  rs274ngc_errors[35] := 'Bug side not right or left';
  rs274ngc_errors[36] := 'Bug unknown motion code';
  rs274ngc_errors[37] := 'Bug unknown operation';
  rs274ngc_errors[38] := 'Cannot change Axis offsets with cutter radius comp';
  rs274ngc_errors[39] := 'Cannot change units with cutter radius comp';
  rs274ngc_errors[40] := 'Cannot create backup file';
  rs274ngc_errors[41] := 'Cannot do g1 with zero feed rate';
  rs274ngc_errors[42] := 'Cannot do zero repeats of cycle';
  rs274ngc_errors[43] := 'Cannot make Arc with zero feed rate';
  rs274ngc_errors[44] := 'Cannot move rotary axes during probing';
  rs274ngc_errors[45] := 'Cannot open backup file';
  rs274ngc_errors[46] := 'Cannot open variable file';
  rs274ngc_errors[47] := 'Cannot probe in inverse time feed Mode';
  rs274ngc_errors[48] := 'Cannot probe with cutter radius comp on';
  rs274ngc_errors[49] := 'Cannot probe with zero feed rate';
  rs274ngc_errors[50] := 'Cannot put A b in canned cycle';
  rs274ngc_errors[51] := 'Cannot put A c in canned cycle';
  rs274ngc_errors[52] := 'Cannot put an A in canned cycle';
  rs274ngc_errors[53] := 'Cannot turn cutter radius comp on out of xy plane';
  rs274ngc_errors[54] := 'Cannot turn cutter radius comp on when on';
  rs274ngc_errors[55] := 'Cannot use A word';
  rs274ngc_errors[56] := 'Cannot use Axis values with g80';
  rs274ngc_errors[57] := 'Cannot use Axis values without A g code that uses them';
  rs274ngc_errors[58] := 'Cannot use b word';
  rs274ngc_errors[59] := 'Cannot use c word';
  rs274ngc_errors[60] := 'Cannot use g28 or g30 with cutter radius comp';
  rs274ngc_errors[61] := 'Cannot use g53 incremental';
  rs274ngc_errors[62] := 'Cannot use g53 with cutter radius comp';
  rs274ngc_errors[63] := 'Cannot use two g codes that both use Axis values';
  rs274ngc_errors[64] := 'Cannot use xz plane with cutter radius comp';
  rs274ngc_errors[65] := 'Cannot use yz plane with cutter radius comp';
  rs274ngc_errors[66] := 'Command too Long';
  rs274ngc_errors[67] := 'Concave corner with cutter radius comp';
  rs274ngc_errors[68] := 'Coordinate system index parameter 5220 out of range';
  rs274ngc_errors[69] := 'Current point same as end point of Arc';
  rs274ngc_errors[70] := 'Cutter gouging with cutter radius comp';
  rs274ngc_errors[71] := 'D word with No g41 or g42';
  rs274ngc_errors[72] := 'Dwell time missing with g4';
  rs274ngc_errors[73] := 'Dwell time p word missing with g82';
  rs274ngc_errors[74] := 'Dwell time p word missing with g86';
  rs274ngc_errors[75] := 'Dwell time p word missing with g88';
  rs274ngc_errors[76] := 'Dwell time p word missing with g89';
  rs274ngc_errors[77] := 'Equal sign missing in parameter setting';
  rs274ngc_errors[78] := 'F word missing with inverse time Arc move';
  rs274ngc_errors[79] := 'F word missing with inverse time g1 move';
  rs274ngc_errors[80] := 'File ended with No percent sign';
  rs274ngc_errors[81] := 'File ended with No percent sign or program end';
  rs274ngc_errors[82] := 'File name too Long';
  rs274ngc_errors[83] := 'File not open';
  rs274ngc_errors[84] := 'g code out of range';
  rs274ngc_errors[85] := 'H word with No g43';
  rs274ngc_errors[86] := 'I word given for Arc in yz plane';
  rs274ngc_errors[87] := 'I word missing with g87';
  rs274ngc_errors[88] := 'I word with No g2 or g3 or g87 to use it';
  rs274ngc_errors[89] := 'J word given for Arc in xz plane';
  rs274ngc_errors[90] := 'J word missing with g87';
  rs274ngc_errors[91] := 'J word with No g2 or g3 or g87 to use it';
  rs274ngc_errors[92] := 'K word given for Arc in xy plane';
  rs274ngc_errors[93] := 'K word missing with g87';
  rs274ngc_errors[94] := 'K word with No g2 or g3 or g87 to use it';
  rs274ngc_errors[95] := 'L word with No canned cycle or g10';
  rs274ngc_errors[96] := 'left bracket missing after slash with atan';
  rs274ngc_errors[97] := 'left bracket missing after unary operation name';
  rs274ngc_errors[98] := 'Line number greater than 99999';
  rs274ngc_errors[99] := 'Line with g10 does not have l2';
  rs274ngc_errors[100] := 'M code greater than 99';
  rs274ngc_errors[101] := 'Mixed radius ijk format for Arc';
  rs274ngc_errors[102] := 'Multiple A words on one Line';
  rs274ngc_errors[103] := 'Multiple b words on one Line';
  rs274ngc_errors[104] := 'Multiple c words on one Line';
  rs274ngc_errors[105] := 'Multiple D words on one Line';
  rs274ngc_errors[106] := 'Multiple F words on one Line';
  rs274ngc_errors[107] := 'Multiple H words on one Line';
  rs274ngc_errors[108] := 'Multiple I words on one Line';
  rs274ngc_errors[109] := 'Multiple J words on one Line';
  rs274ngc_errors[110] := 'Multiple K words on one Line';
  rs274ngc_errors[111] := 'Multiple L words on one Line';
  rs274ngc_errors[112] := 'Multiple p words on one Line';
  rs274ngc_errors[113] := 'Multiple q words on one Line';
  rs274ngc_errors[114] := 'Multiple r words on one Line';
  rs274ngc_errors[115] := 'Multiple s words on one Line';
  rs274ngc_errors[116] := 'Multiple t words on one Line';
  rs274ngc_errors[117] := 'Multiple X words on one Line';
  rs274ngc_errors[118] := 'Multiple Y words on one Line';
  rs274ngc_errors[119] := 'Multiple Z words on one Line';
  rs274ngc_errors[120] := 'Must use g0 or g1 with g53';
  rs274ngc_errors[121] := 'negative Argument to sqrt';
  rs274ngc_errors[122] := 'negative D word tool radius index used';
  rs274ngc_errors[123] := 'negative F word used';
  rs274ngc_errors[124] := 'negative g code used';
  rs274ngc_errors[125] := 'negative H word tool length offset index used';
  rs274ngc_errors[126] := 'negative L word used';
  rs274ngc_errors[127] := 'negative M code used';
  rs274ngc_errors[128] := 'negative or zero q Value used';
  rs274ngc_errors[129] := 'negative p word used';
  rs274ngc_errors[130] := 'negative spindle speed used';
  rs274ngc_errors[131] := 'negative tool id used';
  rs274ngc_errors[132] := 'Nested comment found';
  rs274ngc_errors[133] := 'No characters found in reading real Value';
  rs274ngc_errors[134] := 'No digits found where real number should be';
  rs274ngc_errors[135] := 'non integer Value for integer';
  rs274ngc_errors[136] := 'Null missing after newline';
  rs274ngc_errors[137] := 'offset index missing';
  rs274ngc_errors[138] := 'p Value not an integer with g10 l2';
  rs274ngc_errors[139] := 'p Value out of range with g10 l2';
  rs274ngc_errors[140] := 'p word with No g4 g10 g82 g86 g88 g89';
  rs274ngc_errors[141] := 'parameter file out of order';
  rs274ngc_errors[142] := 'parameter number out of range';
  rs274ngc_errors[143] := 'q word missing with g83';
  rs274ngc_errors[144] := 'q word with No g83';
  rs274ngc_errors[145] := 'Queue is not empty after probing';
  rs274ngc_errors[146] := 'r clearance plane unspecified in cycle';
  rs274ngc_errors[147] := 'r I J K words All missing for Arc';
  rs274ngc_errors[148] := 'r less than X in cycle in yz plane';
  rs274ngc_errors[149] := 'r less than Y in cycle in xz plane';
  rs274ngc_errors[150] := 'r less than Z in cycle in xy plane';
  rs274ngc_errors[151] := 'r word with No g code that uses it';
  rs274ngc_errors[152] := 'radius to end of Arc differs from radius to start';
  rs274ngc_errors[153] := 'radius too small to reach end point';
  rs274ngc_errors[154] := 'Required parameter missing';
  rs274ngc_errors[155] := 'Selected tool slot number too large';
  rs274ngc_errors[156] := 'slash missing after first atan Argument';
  rs274ngc_errors[157] := 'spindle not turning clockwise in g84';
  rs274ngc_errors[158] := 'spindle not turning in g86';
  rs274ngc_errors[159] := 'spindle not turning in g87';
  rs274ngc_errors[160] := 'spindle not turning in g88';
  rs274ngc_errors[161] := 'Sscanf failed';
  rs274ngc_errors[162] := 'start point too close to probe point';
  rs274ngc_errors[163] := 'too many M codes on Line';
  rs274ngc_errors[164] := 'tool length offset index too big';
  rs274ngc_errors[165] := 'tool max too large';
  rs274ngc_errors[166] := 'tool radius index too big';
  rs274ngc_errors[167] := 'tool radius not less than Arc radius with comp';
  rs274ngc_errors[168] := 'two g codes used from same modal group';
  rs274ngc_errors[169] := 'two M codes used from same modal group';
  rs274ngc_errors[170] := 'Unable to open file';
  rs274ngc_errors[171] := 'Unclosed comment found';
  rs274ngc_errors[172] := 'Unclosed expression';
  rs274ngc_errors[173] := 'unknown g code used';
  rs274ngc_errors[174] := 'unknown M code used';
  rs274ngc_errors[175] := 'unknown operation';
  rs274ngc_errors[176] := 'unknown operation name starting with A';
  rs274ngc_errors[177] := 'unknown operation name starting with M';
  rs274ngc_errors[178] := 'unknown operation name starting with o';
  rs274ngc_errors[179] := 'unknown operation name starting with X';
  rs274ngc_errors[180] := 'unknown word starting with A';
  rs274ngc_errors[181] := 'unknown word starting with c';
  rs274ngc_errors[182] := 'unknown word starting with e';
  rs274ngc_errors[183] := 'unknown word starting with F';
  rs274ngc_errors[184] := 'unknown word starting with L';
  rs274ngc_errors[185] := 'unknown word starting with r';
  rs274ngc_errors[186] := 'unknown word starting with s';
  rs274ngc_errors[187] := 'unknown word starting with t';
  rs274ngc_errors[188] := 'unknown word where unary operation could be';
  rs274ngc_errors[189] := 'X and Y words missing for Arc in xy plane';
  rs274ngc_errors[190] := 'X and Z words missing for Arc in xz plane';
  rs274ngc_errors[191] := 'X Value unspecified in yz plane canned cycle';
  rs274ngc_errors[192] := 'X Y and Z words All missing with g38 2';
  rs274ngc_errors[193] := 'Y and Z words missing for Arc in yz plane';
  rs274ngc_errors[194] := 'Y Value unspecified in xz plane canned cycle';
  rs274ngc_errors[195] := 'Z Value unspecified in xy plane canned cycle';
  rs274ngc_errors[196] := 'zero or negative Argument to ln';
  rs274ngc_errors[197] := 'zero radius Arc';
  rs274ngc_errors[198] := 'The End';

end.
