unit FmxUtils;

interface

uses SysUtils, Windows, Classes, Consts;

type
  EInvalidDest = class(EStreamError);
  EFCantMove = class(EStreamError);
  EFReadError = class(EFilerError);
  EFWriteError = class(EFilerError);

function  FileCopy(const SourceName, DestName: string) : Boolean;
procedure MoveFile(const FileName, DestName: string);
function  GetFileSize(const FileName: string): LongInt;
function  HasAttr(const FileName: string; Attr: Word): Boolean;
function  ExecuteFile(const FileName, Params, DefaultDir: string; ShowCmd: Integer): THandle;
procedure RemoveWriteProtect(const FileName : string);

implementation

uses Forms, ShellAPI, RtlConsts;

function FileCopy(const SourceName, DestName: string) : Boolean;
var
  Destination : string;
  Len : integer;
begin
  Try
    Destination := ExpandFileName(DestName); { expand the destination path }
    if HasAttr(Destination, faDirectory) then { if destination is a directory... }
    begin
      Len :=  Length(Destination);
      if Destination[Len] = '\' then
        Destination := Destination + ExtractFileName(SourceName) { ...clone file name }
      else
        Destination := Destination + '\' + ExtractFileName(SourceName); { ...clone file name }
    end;
    Result := CopyFile(PChar(SourceName), PChar(Destination), false);
  Except;
    Result := false;
  End;
end;

{ MoveFile procedure }
{
  Moves the file passed in FileName to the directory specified in DestDir.
  Tries to just rename the file.  If that fails, try to copy the file and
  delete the original.
}

procedure MoveFile(const FileName, DestName: string);
var
  Destination: string;
begin
  Destination := ExpandFileName(DestName); { expand the destination path }
  Try
    if not RenameFile(FileName, Destination) then { try just renaming }
    begin
      RemoveWriteProtect(FileName);    {ensure file is not write protected}
      FileCopy(FileName, Destination); { copy it over to destination...}
  //      DeleteFile(FileName); { ...and delete the original }
    end;
  Except;
  End;  
end;

{ GetFileSize function }
{
  Returns the size of the named file without opening the file.  If the file
  doesn't exist, returns -1.
}

function GetFileSize(const FileName: string): LongInt;
var
  SearchRec: TSearchRec;
begin
  try
    if FindFirst(ExpandFileName(FileName), faAnyFile, SearchRec) = 0 then
      Result := SearchRec.Size
    else
      Result := -1;
  finally
    SysUtils.FindClose(SearchRec);
  end;
end;

function HasAttr(const FileName: string; Attr: Word): Boolean;
var
  FileAttr: Integer;
begin
  try
    FileAttr := FileGetAttr(FileName);
    if FileAttr = -1 then FileAttr := 0;
    Result := (FileAttr and Attr) = Attr;
  Except
    Result := false;
  end;
end;

function ExecuteFile(const FileName, Params, DefaultDir: string;
  ShowCmd: Integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle, nil,
    StrPCopy(zFileName, FileName), StrPCopy(zParams, Params),
    StrPCopy(zDir, DefaultDir), ShowCmd);
end;

procedure RemoveWriteProtect(const FileName : string);
var
  FileAttr: Integer;
begin
  try
    FileAttr := FileGetAttr(FileName);
    FileSetAttr(Filename, FileAttr and not faReadOnly);
  Except
  end;
end;

end.
