unit GetVerInfo;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;

type
  TProjInfo = class(TComponent) 
  private 
    { Private declarations } 
    sFileName , 
    sFileVersion , 
    sFileDescription , 
    sLegalCopyright,
    sComments , 
    sCompanyName , 
    sInternalName , 
    sLegalTrademarks , 
    sOriginalFilename , 
    sProductName , 
    sProductVersion , 
    sLanguage : string; 
    slAllItems: TStrings; 
    procedure SetFileName (exename : string); 
    procedure SetFileVersion (ver : string); 
    procedure SetFileDescription(desc : string); 
    procedure SetLegalCopyright(cr : string); 
    procedure SetComments(cmt : string); 
    procedure SetCompanyName(cn : string); 
    procedure SetInternalName(intn : string); 
    procedure SetLegalTrademarks(tm : string); 
    procedure SetOriginalFilename(ofn : string); 
    procedure SetProductName(pn : string); 
    procedure SetProductVersion(pv : string); 
    procedure SetLanguage(lang : string); 
    procedure SetAllItems(items : Tstrings); 
    procedure GetVersionInfo(AVersionList: TStrings); 
  protected 
    { Protected declarations } 
  public 
    { Public declarations } 
    constructor Create(AOwner: TComponent);  override; 
    procedure execute; 
  published 
    property EXEName : string  read sFileName write SetFileName; 
    property FileVersion: String  read sFileVersion write SetFileVersion; 
    property FileDescription: String  read sFileDescription write SetFileDescription; 
    property LegalCopyright: String  read sLegalCopyright write SetLegalCopyright; 
    property Comments: String  read sComments write SetComments; 
    property CompanyName: String  read sCompanyName write SetCompanyName; 
    property InternalName: String  read sInternalName write SetInternalName; 
    property LegalTrademarks: String  read sLegalTrademarks write SetLegalTrademarks; 
    property OriginalFilename: String  read sOriginalFilename write SetOriginalFilename; 
    property ProductName: String  read sProductName write SetProductName; 
    property ProductVersion: String  read sProductVersion write SetProductVersion; 
    property Language: String  read sLanguage write SetLanguage; 
    property AllItems: TStrings  read slAllItems write SetAllItems; 

    { Published declarations } 
  end; 

procedure Register; 

implementation 

// constructor - creates the needed items
constructor TProjInfo.Create(AOwner: TComponent); 
begin 
  sFileName := '';
  slAllItems   := TStringList.create;
  inherited create(Aowner);
end;

// set the exename property
procedure TProjInfo.SetFileName(exename : string);
begin
  if fileexists(exename) then
    sFileName := exename
  else
    sFileName := '';
end;

{**********************************************}
{**  set the various properties              **}
{**********************************************}

procedure TProjInfo.SetFileVersion(ver: string);
begin
  sFileVersion := ver;
end;

procedure TProjInfo.SetFileDescription(desc : string);
begin
  sFileDescription := desc;
end;

procedure TProjInfo.SetLegalCopyright(cr : string);
begin
  sLegalCopyright := cr;
end;

procedure TProjInfo.SetComments(cmt : string);
begin
  sComments := cmt;
end;

procedure TProjInfo.SetCompanyName(cn : string);
begin
  sCompanyName := cn;
end;

procedure TProjInfo.SetInternalName(intn : string);
begin
  sInternalName := intn;
end;

procedure TProjInfo.SetLegalTrademarks(tm : string);
begin
  sLegalTrademarks := tm;
end;

procedure TProjInfo.SetOriginalFilename(ofn : string);
begin
  sOriginalFilename := ofn;
end;

procedure TProjInfo.SetProductName(pn : string);
begin
  sProductName := pn;
end;

procedure TProjInfo.SetProductVersion(pv : string);
begin
  sProductVersion := pv;
end;

procedure TProjInfo.SetLanguage(lang : string);
begin
  sLanguage := lang;
end;

procedure TProjInfo.SetAllItems(items : Tstrings);
begin
  slAllItems := items;
end;

//  execute method
procedure TProjInfo.execute;
var  alist: TStrings;
begin
  alist := TStringList.create;
  getversioninfo(alist);  // stores the version information into the stringlist
  // only do if information obtained
  if alist.count > 0 then
  begin
    setfileversion(alist[0]);
    SetFileDescription(alist[1]);
    setLegalCopyright(alist[2]);
    SetComments(alist[3]);
    SetCompanyName(alist[4]);
    SetComments(alist[5]);
    SetLegalTrademarks(alist[6]);
    SetOriginalFilename(alist[7]);
    SetProductName(alist[8]);
    SetProductVersion(alist[9]);
    try
      SetLanguage(alist[10]);    // error check in case of failure
    except
    end;
    SetAllItems(alist);
  end;
  alist.free;
end;

procedure TProjInfo.GetVersionInfo(AVersionList: TStrings);
const
  //  all version information items
  CInfoStr : array[1..10] of string =
    ('FileVersion','FileDescription','LegalCopyright','Comments',
     'CompanyName',  'InternalName','LegalTrademarks',
     'OriginalFilename','ProductName', 'ProductVersion');
type
  PTransBuffer = ^TTransBuffer;
  TTransBuffer = array[1..4] of smallint;
var
  transStr,
  typeStr,
  filename,
  language : string;
  i        : integer;
  ptrans   : PTransBuffer;
  value    : PChar;
  verBuf   : pointer;
  infoSize,
  verSize,
  wnd      : DWORD;
  n        : integer;
begin
  // clear the list
  AVersionList.Clear;
  // get application information
  filename := sFileName;
  infoSize := GetFileVersioninfoSize(PChar(filename), wnd);
  // if information exists . . .
  if infoSize <> 0 then
  begin
    GetMem(verBuf, infoSize);
    try
      if GetFileVersionInfo(PChar(filename), wnd, infoSize, verBuf) then
      begin
        VerQueryvalue(verBuf, PChar('\VarFileInfo\Translation'), Pointer(ptrans), verSize);
        transStr := IntToHex(ptrans^[1], 4) + IntToHex(ptrans^[2], 4);

        //  loop through all version information
        N := 0;

        for i := Low(CInfoStr) to High(CInfoStr) do
        begin
          typeStr := 'StringFileInfo\' + transStr + '\' + CInfoStr[i];
          //  get information and store into list
          if VerQueryvalue(verBuf, PChar(typeStr),Pointer(value), verSize) then
          begin
            AVersionList.Add(value);
            inc(N);
          end
          else
            AVersionList.Add(PChar(''));
        end;
        // only add language if the rest completed sucessfully
        if (N > 0) then
        begin
          SetLength(language, 256);
          // get language info and convert to string
          VerLanguageName(ptrans^[1], PChar(language), 256);
          SetLength(language, StrLen(PChar(language)));
          AVersionList.Add(language);
        end;
      end;
    finally
      // memory cleanup
      FreeMem(verBuf);
    end;
  end;
end;

procedure Register; 
begin 
  RegisterComponents('Dockerty', [TProjInfo]); 
end; 

end.
