unit PanelSettings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, ExtCtrls, Vcl.Mask;

type
  TFormPanelSettings = class(TForm)
    ButtonOk: TBitBtn;
    GroupBoxPanelSettings: TGroupBox;
    EditCurrentPanelCount: TLabeledEdit;
    ButtonPanelReset: TButton;
    ButtonGoToNextBoard: TButton;
    EditBoardCountY: TLabeledEdit;
    EditBoardCountX: TLabeledEdit;
    EditBoardSpacingY: TLabeledEdit;
    EditBoardSpacingX: TLabeledEdit;
    ButtonGoToBoard: TButton;
    procedure FormShow(Sender: TObject);
    procedure ButtonPanelResetClick(Sender: TObject);
    procedure ButtonGoToNextBoardClick(Sender: TObject);
    procedure EditCurrentPanelCountKeyPress(Sender: TObject; var Key: Char);
    procedure ButtonGoToBoardClick(Sender: TObject);
    procedure EditBoardSpacingXChange(Sender: TObject);
    procedure EditBoardSpacingYChange(Sender: TObject);
    procedure EditBoardCountXChange(Sender: TObject);
    procedure EditBoardCountYChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    { Private declarations }
    Enabled: boolean;
    procedure UpdateButtons;
    procedure UpdateValues;
  public
    { Public declarations }
  end;

var
  FormPanelSettings: TFormPanelSettings;

implementation

{$R *.dfm}

uses
  Main, Settings, DeskWinCNC;

procedure TFormPanelSettings.UpdateButtons;
begin
  ButtonGoToBoard.Enabled := CNCControllerFound and CNCInitialised and not CNCEmergencyStop;
  ButtonGoToNextBoard.Enabled := CNCControllerFound and CNCInitialised and not CNCEmergencyStop;
end;

procedure TFormPanelSettings.UpdateValues;
begin
  Enabled := false;

  if BoardMetric then
  begin
    EditBoardSpacingX.Text := FloatToStrF(PanelSpacingX, ffFixed, 10, 3);
    EditBoardSpacingY.Text := FloatToStrF(PanelSpacingY, ffFixed, 10, 3);
  end
  else
  begin
    EditBoardSpacingX.Text := FloatToStrF(PanelSpacingX / 25.4, ffFixed, 10, 4);
    EditBoardSpacingY.Text := FloatToStrF(PanelSpacingY / 25.4, ffFixed, 10, 4);
  end;

  if PanelCountX <= 0 then
    PanelCountX := 1; // range check
  EditBoardCountX.Text := IntToStr(PanelCountX);

  if PanelCountY <= 0 then
    PanelCountY := 1; // range check
  EditBoardCountY.Text := IntToStr(PanelCountY);

  if PanelBoardCount <= 0 then
    PanelBoardCount := 1; // range check

  EditCurrentPanelCount.Text := IntToStr(PanelBoardCount);

  Enabled := true;
end;

procedure TFormPanelSettings.ButtonGoToBoardClick(Sender: TObject);
begin
  FormMain.MoveHomeRelativeWithNozzleProtection(BoardOriginX, BoardOriginY, 0, 0, CNCMaxStepsPerSecond);
  // move to position

  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
  UpdateValues;
  UpdateButtons;
end;

procedure TFormPanelSettings.ButtonGoToNextBoardClick(Sender: TObject);
begin
  Inc(PanelBoardCount); // advance board count
  if PanelBoardCount > PanelCountX * PanelCountY then
    PanelBoardCount := 1; // wrap around
  CurrentPart := 1;

  FormMain.UpdatePartList;
  FormMain.UpdateStatus;
  FormMain.UpdateButtons;

  FormMain.MoveHomeRelativeWithNozzleProtection(BoardOriginX, BoardOriginY, 0, 0, CNCMaxStepsPerSecond);
  // move to position

  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
  UpdateValues;
  UpdateButtons;
end;

procedure TFormPanelSettings.ButtonPanelResetClick(Sender: TObject);
begin
  PanelBoardCount := 1; // reset panel
  CurrentPart := 1;

  FormMain.UpdatePartList;
  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
  UpdateValues;
  UpdateButtons;
end;

procedure TFormPanelSettings.EditCurrentPanelCountKeyPress(Sender: TObject; var Key: Char);
begin
  Key := #0; // cancel keystrokes
end;

procedure TFormPanelSettings.EditBoardCountXChange(Sender: TObject);
Var
  V: integer;
begin
  if Enabled then
  begin
    try
      V := StrToInt(EditBoardCountX.Text);
      if (V > 0) and (V <= 50) then
      begin
        EditBoardCountX.Color := clWindow;
        PanelCountX := V;
        ProjectEdited := true;
      end
      else
        EditBoardCountX.Color := clRed;
    except
      PanelCountX := 1;
      EditBoardCountX.Color := clRed;
    end;
  end;
end;

procedure TFormPanelSettings.EditBoardCountYChange(Sender: TObject);
Var
  V: integer;
begin
  if Enabled then
  begin
    try
      V := StrToInt(EditBoardCountY.Text);
      if (V > 0) and (V <= 50) then
      begin
        EditBoardCountY.Color := clWindow;
        PanelCountY := V;
        ProjectEdited := true;
      end
      else
        EditBoardCountY.Color := clRed;
    except
      PanelCountY := 1;
      EditBoardCountY.Color := clRed;
    end;
  end;
end;

procedure TFormPanelSettings.EditBoardSpacingXChange(Sender: TObject);
Var
  V: double;
begin
  if Enabled then
  begin
    try
      if BoardMetric then
        V := StrToFloat(EditBoardSpacingX.Text)
      else
        V := StrToFloat(EditBoardSpacingX.Text) * 25.4; // convert to metric

      if (V >= 0) and (V <= 500) then
      begin
        EditBoardSpacingX.Color := clWindow;
        PanelSpacingX := V;
        ProjectEdited := true;
      end
      else
        EditBoardSpacingX.Color := clRed;
    except
      EditBoardSpacingX.Color := clRed;
    end;
  end;
end;

procedure TFormPanelSettings.EditBoardSpacingYChange(Sender: TObject);
Var
  V: double;
begin
  if Enabled then
  begin
    try
      if BoardMetric then
        V := StrToFloat(EditBoardSpacingY.Text)
      else
        V := StrToFloat(EditBoardSpacingY.Text) * 25.4; // convert to metric

      if (V >= 0) and (V <= 500) then
      begin
        EditBoardSpacingY.Color := clWindow;
        PanelSpacingY := V;
        ProjectEdited := true;
      end
      else
        EditBoardSpacingY.Color := clRed;
    except
      EditBoardSpacingY.Color := clRed;
    end;
  end;
end;

procedure TFormPanelSettings.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  Save_Settings; // ensure the settings are saved first
end;

procedure TFormPanelSettings.FormShow(Sender: TObject);
begin
  Self.Left := Screen.Width - Self.Width; // position at top right
  Self.Top := 0;

  ViewSelection := 0; // downlooking camera
  UpdateValues;
  UpdateButtons;
end;

end.
