unit VisionSettings;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Image, Settings, ExtCtrls, UiTypes, WCamera, Vcl.Mask;

type
  TFormVisionSettings = class(TForm)
    ButtonClose: TBitBtn;
    GroupBoxCameraSelection: TGroupBox;
    RadioButtonUpCamera: TRadioButton;
    RadioButtonDownCamera: TRadioButton;
    GroupBoxDownCamera: TGroupBox;
    GroupBoxUpCamera: TGroupBox;
    EditDownlookingViewWidthInMM: TLabeledEdit;
    EditUplookingViewWidthInMM: TLabeledEdit;
    EditPositionY: TLabeledEdit;
    EditPositionX: TLabeledEdit;
    ButtonGotoCamera: TButton;
    ButtonSetPosition: TButton;
    EditDownlookingSettlingDelay: TLabeledEdit;
    EditUplookingSettlingDelay: TLabeledEdit;
    EditPositionZ: TLabeledEdit;
    Label1: TLabel;
    EditUplookingEnlargementMM: TLabeledEdit;
    EditAngleTestLimit: TLabeledEdit;
    EditDownlookingEnlargementMM: TLabeledEdit;
    CheckBoxVisionTestMode: TCheckBox;
    EditVisionCorrelationLimit: TLabeledEdit;
    ButtonDownCameraStart: TBitBtn;
    ButtonDownCameraStop: TBitBtn;
    ButtonUpCameraStop: TBitBtn;
    ButtonUpCameraStart: TBitBtn;
    GroupBoxJogControl: TGroupBox;
    ButtonJogZMinus: TButton;
    ButtonJogZPlus: TButton;
    ButtonJogXMinus: TButton;
    ButtonJogYMinus: TButton;
    ButtonJogYPlus: TButton;
    ButtonJogXPlus: TButton;
    ButtonZMinus1: TButton;
    ButtonZPlus1: TButton;
    ButtonYMinus1: TButton;
    ButtonYPlus1: TButton;
    ButtonXMinus1: TButton;
    ButtonXPlus1: TButton;
    ButtonZMinus10: TButton;
    ButtonZPlus10: TButton;
    ButtonYMinus10: TButton;
    ButtonYPlus10: TButton;
    ButtonXMinus10: TButton;
    ButtonXPlus10: TButton;
    ComboBoxCameraUp: TComboBox;
    ComboBoxCameraDown: TComboBox;
    WCameraDown: TWCamera;
    WCameraUp: TWCamera;
    ComboBoxFormatDown: TComboBox;
    ComboBoxFormatUp: TComboBox;
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure EditDownlookingViewWidthInMMChange(Sender: TObject);
    procedure EditUplookingViewWidthInMMChange(Sender: TObject);
    procedure RadioButtonUpCameraClick(Sender: TObject);
    procedure RadioButtonDownCameraClick(Sender: TObject);
    procedure ButtonGotoCameraClick(Sender: TObject);
    procedure ButtonSetPositionClick(Sender: TObject);
    procedure EditPositionXChange(Sender: TObject);
    procedure EditPositionYChange(Sender: TObject);
    procedure EditDownlookingSettlingDelayChange(Sender: TObject);
    procedure EditUplookingSettlingDelayChange(Sender: TObject);
    procedure ButtonXPlus10Click(Sender: TObject);
    procedure ButtonXPlus1Click(Sender: TObject);
    procedure ButtonJogXPlusClick(Sender: TObject);
    procedure ButtonJogXMinusClick(Sender: TObject);
    procedure ButtonXMinus1Click(Sender: TObject);
    procedure ButtonXMinus10Click(Sender: TObject);
    procedure ButtonYPlus10Click(Sender: TObject);
    procedure ButtonYPlus1Click(Sender: TObject);
    procedure ButtonJogYPlusClick(Sender: TObject);
    procedure ButtonJogYMinusClick(Sender: TObject);
    procedure ButtonYMinus1Click(Sender: TObject);
    procedure ButtonYMinus10Click(Sender: TObject);
    procedure ButtonZPlus10Click(Sender: TObject);
    procedure ButtonZPlus1Click(Sender: TObject);
    procedure ButtonJogZPlusClick(Sender: TObject);
    procedure ButtonJogZMinusClick(Sender: TObject);
    procedure ButtonZMinus1Click(Sender: TObject);
    procedure ButtonZMinus10Click(Sender: TObject);
    procedure EditPositionZChange(Sender: TObject);
    procedure EditUplookingEnlargementMMChange(Sender: TObject);
    procedure EditAngleTestLimitChange(Sender: TObject);
    procedure EditDownlookingEnlargementMMChange(Sender: TObject);
    procedure CheckBoxVisionTestModeClick(Sender: TObject);
    procedure EditVisionCorrelationLimitChange(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ButtonDownCameraStopClick(Sender: TObject);
    procedure ButtonUpCameraStartClick(Sender: TObject);
    procedure ButtonUpCameraStopClick(Sender: TObject);
    procedure ComboBoxCameraDownDropDown(Sender: TObject);
    procedure ComboBoxCameraUpDropDown(Sender: TObject);
    procedure ComboBoxCameraUpChange(Sender: TObject);
    procedure ComboBoxCameraDownChange(Sender: TObject);
    procedure ButtonDownCameraStartClick(Sender: TObject);
    procedure WCameraUpImageAvailable(Sender: TObject; SampleTime: Double);
    procedure WCameraDownImageAvailable(Sender: TObject; SampleTime: Double);
  private
    { Private declarations }
    Enabled: boolean;
    OldViewSelection: integer;
    FriendlyDeviceList: TStringList;
    procedure UpdateValues;
    procedure UpdateButtons;
  public
    { Public declarations }

    MemoryStreamUp, MemoryStreamDown: TMemoryStream;
    BitmapUp, BitmapDown: TBitmap;
    FormatUp, FormatDown: TWCameraFormat;
    CameraDeviceNameUp, CameraDeviceNameDown: string;
    CameraFormatNameUp, CameraFormatNameDown: string;
    VideoFormatsUp, VideoFormatsDown: TStringList; // complete list of video formats before filtering

    procedure SetDevicesUp;
    procedure SetDevicesDown;
    procedure SetFormatsUp;
    procedure SetFormatsDown;
    procedure StartDeviceUp;
    procedure StartDeviceDown;
    procedure UpdateVideoPositionUp;
    procedure UpdateVideoPositionDown;
    procedure StopCameraUp;
    procedure StopCameraDown;
    procedure StartCameraUp;
    procedure StartCameraDown;
    procedure StartCameras;

  end;

var
  FormVisionSettings: TFormVisionSettings;

implementation

uses Main, DeskWinCNC, StrUtils, WinApi.DirectShow9;

{$R *.dfm}

procedure TFormVisionSettings.SetDevicesUp;
var
  Devices: TWVideoCaptureDevices;
  I: integer;
begin
  Devices := nil;

  ComboBoxCameraUp.Items.BeginUpdate;
  try
    Devices := WCameraUp.Devices;
    ComboBoxCameraUp.Items.Clear;
    for I := 0 to Length(Devices) - 1 do
      ComboBoxCameraUp.Items.Add(Devices[I].Name);
  finally
    ComboBoxCameraUp.Items.EndUpdate;
  end;
end;

procedure TFormVisionSettings.SetDevicesDown;
var
  Devices: TWVideoCaptureDevices;
  I: integer;
begin
  Devices := nil;

  ComboBoxCameraDown.Items.BeginUpdate;
  try
    Devices := WCameraDown.Devices;
    ComboBoxCameraDown.Items.Clear;
    for I := 0 to Length(Devices) - 1 do
      ComboBoxCameraDown.Items.Add(Devices[I].Name);
  finally
    ComboBoxCameraDown.Items.EndUpdate;
  end;
end;

procedure TFormVisionSettings.SetFormatsUp;
var
  Formats: TWCameraFormats;
  I: integer;
begin
  Formats := nil;

  ComboBoxFormatUp.Items.BeginUpdate;
  try
    WCameraUp.Active := False;
    WCameraUp.DeviceName := ComboBoxCameraUp.Text;
    ComboBoxFormatUp.Items.Clear;
    ComboBoxFormatUp.Enabled := WCameraUp.DeviceName <> '';
    if ComboBoxFormatUp.Enabled then
    begin
      Formats := WCameraUp.SupportedFormats;
      for I := 0 to Length(Formats) - 1 do
        if Formats[I].AvgTimePerFrame = 0 then
          ComboBoxFormatUp.Items.Add(IntToStr(Formats[I].Width) + 'x' + IntToStr(Formats[I].Height) + ' ' + IntToStr(Formats[I].BitsPerPixel) + 'bpp')
        else
          ComboBoxFormatUp.Items.Add(IntToStr(Formats[I].Width) + 'x' + IntToStr(Formats[I].Height) + ' ' + IntToStr(10000000 div Formats[I].AvgTimePerFrame) +
            'Hz ' + IntToStr(Formats[I].BitsPerPixel) + 'bpp');
    end
  finally
    ComboBoxFormatUp.Items.EndUpdate;
  end;
end;

procedure TFormVisionSettings.SetFormatsDown;
var
  Formats: TWCameraFormats;
  I: integer;
begin
  Formats := nil;

  ComboBoxFormatDown.Items.BeginUpdate;
  try
    WCameraDown.Active := False;
    WCameraDown.DeviceName := ComboBoxCameraDown.Text;
    ComboBoxFormatDown.Items.Clear;
    ComboBoxFormatDown.Enabled := WCameraDown.DeviceName <> '';
    if ComboBoxFormatDown.Enabled then
    begin
      Formats := WCameraDown.SupportedFormats;
      for I := 0 to Length(Formats) - 1 do
        if Formats[I].AvgTimePerFrame = 0 then
          ComboBoxFormatDown.Items.Add(IntToStr(Formats[I].Width) + 'x' + IntToStr(Formats[I].Height) + ' ' + IntToStr(Formats[I].BitsPerPixel) + 'bpp')
        else
          ComboBoxFormatDown.Items.Add(IntToStr(Formats[I].Width) + 'x' + IntToStr(Formats[I].Height) + ' ' + IntToStr(10000000 div Formats[I].AvgTimePerFrame)
            + 'Hz ' + IntToStr(Formats[I].BitsPerPixel) + 'bpp');
    end
  finally
    ComboBoxFormatDown.Items.EndUpdate;
  end;
end;

procedure TFormVisionSettings.StartDeviceUp;
begin
  if ComboBoxFormatUp.ItemIndex <> -1 then
  begin
    WCameraUp.Active := False;
    try
      FormatUp := WCameraUp.SupportedFormats[ComboBoxFormatUp.ItemIndex];
      WCameraUp.Format := FormatUp;
      WCameraUp.Active := True;
      UpdateVideoPositionUp;
      WCameraUp.Run;
    except
    end;
  end
end;

procedure TFormVisionSettings.StartDeviceDown;
begin
  if ComboBoxFormatDown.ItemIndex <> -1 then
  begin
    try
      WCameraDown.Active := False;
      FormatDown := WCameraDown.SupportedFormats[ComboBoxFormatDown.ItemIndex];
      WCameraDown.Format := FormatDown;
      WCameraDown.Active := True;
      UpdateVideoPositionDown;
      WCameraDown.Run;
    except
    end;
  end
end;

procedure TFormVisionSettings.UpdateVideoPositionUp;
begin
  if WCameraUp.Active and (WCameraUp.CaptureType = ctVmr9) then
  begin
    WCameraUp.VideoPositionDest := FormMain.PanelCameraUp.ClientRect;
    FormMain.PanelCameraUp.Invalidate;
  end;
end;

procedure TFormVisionSettings.WCameraDownImageAvailable(Sender: TObject; SampleTime: Double);
begin
  UpdateVideoPositionDown;
end;

procedure TFormVisionSettings.WCameraUpImageAvailable(Sender: TObject; SampleTime: Double);
begin
  UpdateVideoPositionUp;
end;

procedure TFormVisionSettings.UpdateVideoPositionDown;
begin
  if WCameraDown.Active and (WCameraDown.CaptureType = ctVmr9) then
  begin
    WCameraDown.VideoPositionDest := FormMain.PanelCameraDown.ClientRect;
    FormMain.PanelCameraDown.Invalidate;
  end;
end;

procedure TFormVisionSettings.StopCameraUp;
begin
  try
    FormMain.PanelCameraUp.Visible := False;
    WCameraUp.Active := False;
  except
  end;
end;

procedure TFormVisionSettings.StopCameraDown;
begin
  try
    FormMain.PanelCameraDown.Visible := False;
    WCameraDown.Active := False;
  except
  end;
end;

procedure TFormVisionSettings.StartCameraUp;
begin
  StartDeviceUp;

  if WCameraUp.Active then
  begin
    // **************** if success then save new settings **************************
    CameraDeviceNameUp := ComboBoxCameraUp.Items[ComboBoxCameraUp.ItemIndex]; // set camera
    CameraFormatNameUp := ComboBoxFormatUp.Items[ComboBoxFormatUp.ItemIndex]; // set format
  end;
end;

procedure TFormVisionSettings.StartCameraDown;
begin
  StartDeviceDown;

  if WCameraDown.Active then
  begin
    // **************** if success then save new settings **************************
    CameraDeviceNameDown := ComboBoxCameraDown.Items[ComboBoxCameraDown.ItemIndex]; // set camera
    CameraFormatNameDown := ComboBoxFormatDown.Items[ComboBoxFormatDown.ItemIndex]; // set format
  end;
end;

procedure TFormVisionSettings.UpdateValues;
begin
  Enabled := False;

  EditDownlookingViewWidthInMM.Text := FloatToStrF(DownlookingViewWidthInMM, ffFixed, 10, 3);
  EditUplookingViewWidthInMM.Text := FloatToStrF(UplookingViewWidthInMM, ffFixed, 10, 3);

  EditPositionX.Text := FloatToStrF(UplookingCameraX, ffFixed, 10, 3);
  EditPositionY.Text := FloatToStrF(UplookingCameraY, ffFixed, 10, 3);
  EditPositionZ.Text := FloatToStrF(UplookingCameraZ, ffFixed, 10, 3);
  EditDownlookingSettlingDelay.Text := FloatToStrF(DownlookingCameraSettlingDelay, ffFixed, 10, 3);
  EditDownlookingEnlargementMM.Text := IntToStr(DownlookingCameraEnlargementMM);
  EditUplookingSettlingDelay.Text := FloatToStrF(UplookingCameraSettlingDelay, ffFixed, 10, 3);
  EditUplookingEnlargementMM.Text := IntToStr(UplookingCameraEnlargementMM);
  EditAngleTestLimit.Text := IntToStr(UplookingCameraAngleTestLimit);
  EditVisionCorrelationLimit.Text := FloatToStrF(VisionCorrelationLimit, ffFixed, 10, 2);

  CheckBoxVisionTestMode.Checked := VisionTestMode;

  Enabled := True;
end;

procedure TFormVisionSettings.StartCameras;
begin
  SetDevicesUp;
  SetDevicesDown;

  ComboBoxCameraUp.ItemIndex := ComboBoxCameraUp.Items.IndexOf(CameraDeviceNameUp);
  ComboBoxCameraDown.ItemIndex := ComboBoxCameraDown.Items.IndexOf(CameraDeviceNameDown);

  SetFormatsUp;
  SetFormatsDown;

  ComboBoxFormatUp.ItemIndex := ComboBoxFormatUp.Items.IndexOf(CameraFormatNameUp);
  ComboBoxFormatDown.ItemIndex := ComboBoxFormatDown.Items.IndexOf(CameraFormatNameDown);

  ViewSelection := 1; // uplooking camera
  FormMain.UpdateButtons;

  if (ComboBoxFormatUp.ItemIndex >= 0) then
    StartCameraUp;

  ViewSelection := 0; // downlooking camera
  FormMain.UpdateButtons;

  if (ComboBoxCameraDown.ItemIndex >= 0) and (ComboBoxFormatDown.ItemIndex >= 0) then
    StartCameraDown;

  UpdateButtons;
  FormMain.UpdateButtons;
end;

procedure TFormVisionSettings.UpdateButtons;
begin
  ButtonGotoCamera.Enabled := CNCControllerFound and CNCInitialised and not CNCEmergencyStop;
  ButtonSetPosition.Enabled := CNCControllerFound and CNCInitialised and not CNCEmergencyStop;
  GroupBoxJogControl.Enabled := CNCControllerFound and CNCInitialised and not CNCEmergencyStop;

  ButtonUpCameraStart.Enabled := not WCameraUp.Active;
  ButtonUpCameraStop.Enabled := WCameraUp.Active;

  ButtonDownCameraStart.Enabled := not WCameraDown.Active;
  ButtonDownCameraStop.Enabled := WCameraDown.Active;

  ComboBoxCameraUp.Enabled := not WCameraUp.Active;
  ComboBoxFormatUp.Enabled := not WCameraUp.Active;
  ComboBoxFormatDown.Enabled := not WCameraDown.Active;
  ComboBoxCameraDown.Enabled := not WCameraDown.Active;

  Enabled := False;

  case ViewSelection of // select required view
    0: // downlooking
      begin
        RadioButtonDownCamera.Checked := True;
      end;
    1: // uplooking
      begin
        RadioButtonUpCamera.Checked := True;
      end;
  end;
  Enabled := True;
end;

procedure TFormVisionSettings.ButtonGotoCameraClick(Sender: TObject);
begin
  ViewSelection := 1; // uplooking camera
  FormMain.UpdateStatus;
  UpdateButtons;

  FormMain.MoveHomeRelativeWithNozzleProtection(UplookingCameraX, UplookingCameraY, UplookingCameraZ, 0, CNCMaxStepsPerSecond);

  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
  UpdateButtons;
end;

procedure TFormVisionSettings.ButtonSetPositionClick(Sender: TObject);
begin
  ViewSelection := 1; // uplooking camera
  FormMain.UpdateStatus;
  UpdateButtons;

  if MessageDlg('ARE YOU SURE YOU WANT TO CHANGE THE CAMERA POSITION?', mtConfirmation, [mbYes, mbNo], 0, mbYes) = mrYes then
  begin
    UplookingCameraX := CNCXPos - CNCActualHomeX;
    UplookingCameraY := CNCYPos - CNCActualHomeY;
    UpdateValues;
  end;

  if (UplookingCameraZ <> CNCZPos - CNCHomeZ) and
    (MessageDlg('"WARNING!! THE CAMERA HEIGHT HAS CHANGED' + #$0d + 'ARE YOU SURE YOU WANT TO' + #$0d + 'CHANGE THE CAMERA HEIGHT?', mtWarning, [mbYes, mbNo],
    0, mbYes) = mrYes) and (MessageDlg('LAST WARNING!!!' + #$0d + #$0d + 'DO YOU REALLY WANT TO' + #$0d + 'CHANGE THE CAMERA HEIGHT?', mtWarning, [mbYes, mbNo],
    0, mbYes) = mrYes) then
  begin
    UplookingCameraZ := CNCZPos - CNCHomeZ;
    UpdateValues;
  end;

  UpdateButtons;
end;

procedure TFormVisionSettings.EditAngleTestLimitChange(Sender: TObject);
var
  V: integer;
begin
  if Enabled then
  begin
    try
      V := StrToInt(EditAngleTestLimit.Text);
      if (V > 10) and (V < 90) then // restrict range
      begin
        UplookingCameraAngleTestLimit := V;
        EditAngleTestLimit.Color := clWindow; // normal colour
      end
      else
        EditAngleTestLimit.Color := clRed; // error colour
    except
      EditAngleTestLimit.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditDownlookingEnlargementMMChange(Sender: TObject);
var
  V: integer;
begin
  if Enabled then
  begin
    try
      V := StrToInt(EditDownlookingEnlargementMM.Text);
      if (V > 0) and (V < 10) then // restrict range
      begin
        DownlookingCameraEnlargementMM := V;
        EditDownlookingEnlargementMM.Color := clWindow; // normal colour
      end
      else
        EditDownlookingEnlargementMM.Color := clRed; // error colour
    except
      EditDownlookingEnlargementMM.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditDownlookingSettlingDelayChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditDownlookingSettlingDelay.Text);
      if (V > 0) and (V < 5) then // restrict range
      begin
        DownlookingCameraSettlingDelay := V;
        EditDownlookingSettlingDelay.Color := clWindow; // normal colour
      end
      else
        EditDownlookingSettlingDelay.Color := clRed; // error colour
    except
      EditDownlookingSettlingDelay.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditDownlookingViewWidthInMMChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditDownlookingViewWidthInMM.Text);
      if (V > 1) and (V < 100) then // restrict range
      begin
        DownlookingViewWidthInMM := V;
        EditDownlookingViewWidthInMM.Color := clWindow; // normal colour
        FormMain.UpdateVideoGridRulers;
      end
      else
        EditDownlookingViewWidthInMM.Color := clRed; // error colour
    except
      EditDownlookingViewWidthInMM.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditUplookingEnlargementMMChange(Sender: TObject);
var
  V: integer;
begin
  if Enabled then
  begin
    try
      V := StrToInt(EditUplookingEnlargementMM.Text);
      if (V > 0) and (V < 10) then // restrict range
      begin
        UplookingCameraEnlargementMM := V;
        EditUplookingEnlargementMM.Color := clWindow; // normal colour
      end
      else
        EditUplookingEnlargementMM.Color := clRed; // error colour
    except
      EditUplookingEnlargementMM.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditPositionXChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditPositionX.Text);
      if (V >= CNCMinX - CNCActualHomeX) and (V <= CNCMaxX - CNCActualHomeX) then // restrict range
      begin
        UplookingCameraX := V;
        EditPositionX.Color := clWindow; // normal colour
      end
      else
        EditPositionX.Color := clRed; // error colour
    except
      EditPositionX.Color := clRed; // error colour
    end;
  end;
  UpdateButtons;
end;

procedure TFormVisionSettings.EditPositionYChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditPositionY.Text);
      if (V >= CNCMinY - CNCActualHomeY) and (V <= CNCMaxY - CNCActualHomeY) then // restrict range
      begin
        UplookingCameraY := V;
        EditPositionY.Color := clWindow; // normal colour
      end
      else
        EditPositionY.Color := clRed; // error colour
    except
      EditPositionY.Color := clRed; // error colour
    end;
  end;
  UpdateButtons;
end;

procedure TFormVisionSettings.EditPositionZChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditPositionZ.Text);
      if (V >= CNCMinZ - CNCHomeZ) and (V <= CNCMaxZ - CNCHomeZ) then // restrict range
      begin
        UplookingCameraZ := V;
        EditPositionZ.Color := clWindow; // normal colour
      end
      else
        EditPositionZ.Color := clRed; // error colour
    except
      EditPositionZ.Color := clRed; // error colour
    end;
  end;
  UpdateButtons;
end;

procedure TFormVisionSettings.EditUplookingSettlingDelayChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditUplookingSettlingDelay.Text);
      if (V > 0) and (V < 5) then // restrict range
      begin
        UplookingCameraSettlingDelay := V;
        EditUplookingSettlingDelay.Color := clWindow; // normal colour
      end
      else
        EditUplookingSettlingDelay.Color := clRed; // error colour
    except
      EditUplookingSettlingDelay.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditUplookingViewWidthInMMChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditUplookingViewWidthInMM.Text);
      if (V > 1) and (V < 100) then // restrict range
      begin
        UplookingViewWidthInMM := V;
        EditUplookingViewWidthInMM.Color := clWindow; // normal colour
        FormMain.UpdateVideoGridRulers;
      end
      else
        EditUplookingViewWidthInMM.Color := clRed; // error colour
    except
      EditUplookingViewWidthInMM.Color := clRed; // error colour
    end;
  end;
end;

procedure TFormVisionSettings.EditVisionCorrelationLimitChange(Sender: TObject);
var
  V: Double;
begin
  if Enabled then
  begin
    try
      V := StrToFloat(EditVisionCorrelationLimit.Text);
      if (V >= 0.3) and (V <= 0.9) then // restrict range
      begin
        VisionCorrelationLimit := V;
        EditVisionCorrelationLimit.Color := clWindow; // normal colour
      end
      else
        EditVisionCorrelationLimit.Color := clRed; // error colour
    except
      EditVisionCorrelationLimit.Color := clRed; // error colour
    end;
  end;
  UpdateButtons;
end;

procedure TFormVisionSettings.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  ViewSelection := OldViewSelection; // restore view
  FormMain.UpdateButtons;
  Enabled := False;
  Save_Settings; // ensure the settings are saved first
end;

procedure TFormVisionSettings.FormCreate(Sender: TObject);
begin
  Enabled := False;
  FriendlyDeviceList := TStringList.Create;

  MemoryStreamUp := TMemoryStream.Create;
  MemoryStreamDown := TMemoryStream.Create;

  BitmapUp := TBitmap.Create;
  BitmapDown := TBitmap.Create;

  VideoFormatsUp := TStringList.Create;
  VideoFormatsDown := TStringList.Create;

end;

procedure TFormVisionSettings.FormDestroy(Sender: TObject);
begin
  FriendlyDeviceList.Free;

  try
    WCameraUp.Active := False;
    WCameraDown.Active := False;

    BitmapUp.Free;
    BitmapDown.Free;

    MemoryStreamUp.Free;
    MemoryStreamDown.Free;

  except
  end;

end;

procedure TFormVisionSettings.FormShow(Sender: TObject);
begin
  Enabled := False;

  Self.Left := Screen.Width - Self.Width; // position at top right
  Self.Top := 0;

  OldViewSelection := ViewSelection; // preserve view

  case ViewSelection of // select required view
    0: // downlooking
      begin
        RadioButtonDownCamera.Checked := True;
      end;
    1: // uplooking
      begin
        RadioButtonUpCamera.Checked := True;
      end;
  end;
  Enabled := True;

  UpdateValues;
  UpdateButtons;
  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
end;

procedure TFormVisionSettings.RadioButtonDownCameraClick(Sender: TObject);
begin
  if Enabled then
    ViewSelection := 0; // downlooking camera

  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
  UpdateButtons;
end;

procedure TFormVisionSettings.RadioButtonUpCameraClick(Sender: TObject);
begin
  if Enabled then
    ViewSelection := 1; // uplooking camera

  FormMain.UpdateStatus;
  FormMain.UpdateButtons;
  UpdateButtons;
end;

procedure TFormVisionSettings.ButtonXPlus1Click(Sender: TObject);
begin
  JogX := 1;
  JogY := 0;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonYMinus10Click(Sender: TObject);
begin
  JogX := 0;
  JogY := -10;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonYMinus1Click(Sender: TObject);
begin
  JogX := 0;
  JogY := -1;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonYPlus10Click(Sender: TObject);
begin
  JogX := 0;
  JogY := 10;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonYPlus1Click(Sender: TObject);
begin
  JogX := 0;
  JogY := 1;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonZMinus10Click(Sender: TObject);
begin
  JogX := 0;
  JogY := 0;
  JogZ := -10;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonZMinus1Click(Sender: TObject);
begin
  JogX := 0;
  JogY := 0;
  JogZ := -1;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonZPlus10Click(Sender: TObject);
begin
  JogX := 0;
  JogY := 0;
  JogZ := 10;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonZPlus1Click(Sender: TObject);
begin
  JogX := 0;
  JogY := 0;
  JogZ := 1;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.CheckBoxVisionTestModeClick(Sender: TObject);
begin
  if Enabled then
    VisionTestMode := CheckBoxVisionTestMode.Checked;
end;

procedure TFormVisionSettings.ComboBoxCameraDownChange(Sender: TObject);
begin
  SetFormatsDown;
end;

procedure TFormVisionSettings.ComboBoxCameraDownDropDown(Sender: TObject);
begin
  SetDevicesDown;
end;

procedure TFormVisionSettings.ComboBoxCameraUpChange(Sender: TObject);
begin
  SetFormatsUp;
end;

procedure TFormVisionSettings.ComboBoxCameraUpDropDown(Sender: TObject);
begin
  SetDevicesUp;
end;

procedure TFormVisionSettings.ButtonUpCameraStartClick(Sender: TObject);
begin
  ViewSelection := 1; // uplooking camera
  StartCameraUp;
  UpdateButtons;
  FormMain.UpdateButtons;
end;

procedure TFormVisionSettings.ButtonUpCameraStopClick(Sender: TObject);
begin
  StopCameraUp;
  UpdateButtons;
end;

procedure TFormVisionSettings.ButtonDownCameraStartClick(Sender: TObject);
begin
  ViewSelection := 0; // downlooking camera
  StartCameraDown;
  UpdateButtons;
  FormMain.UpdateButtons;
end;

procedure TFormVisionSettings.ButtonDownCameraStopClick(Sender: TObject);
begin
  StopCameraDown;
  UpdateButtons;
end;

procedure TFormVisionSettings.ButtonJogXMinusClick(Sender: TObject);
begin
  JogX := -JOG_SMALL_STEP;
  JogY := 0;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonJogXPlusClick(Sender: TObject);
begin
  JogX := JOG_SMALL_STEP;
  JogY := 0;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonJogYMinusClick(Sender: TObject);
begin
  JogX := 0;
  JogY := -JOG_SMALL_STEP;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonJogYPlusClick(Sender: TObject);
begin
  JogX := 0;
  JogY := JOG_SMALL_STEP;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonJogZMinusClick(Sender: TObject);
begin
  JogX := 0;
  JogY := 0;
  JogZ := -JOG_SMALL_STEP;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonJogZPlusClick(Sender: TObject);
begin
  JogX := 0;
  JogY := 0;
  JogZ := JOG_SMALL_STEP;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonXMinus10Click(Sender: TObject);
begin
  JogX := -10;
  JogY := 0;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonXMinus1Click(Sender: TObject);
begin
  JogX := -1;
  JogY := 0;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

procedure TFormVisionSettings.ButtonXPlus10Click(Sender: TObject);
begin
  JogX := 10;
  JogY := 0;
  JogZ := 0;
  JogA := 0;
  Jog;
  FormMain.UpdateStatus;
end;

end.
